/**
 * @file include/gbGraphicsObject.hpp
 *
 * ghulbusGraphics - The Ghulbus Graphics Library
 *
 * @brief gbGraphics Definition of gbGraphicsObject Interface
 *
 * @version 0.1
 * @author Andreas Weis
 *
 */

#ifndef _GHULBUSGRAPHICS_GRAPHICSOBJECT_HPP_INCLUDE_GUARD_
#define _GHULBUSGRAPHICS_GRAPHICSOBJECT_HPP_INCLUDE_GUARD_

namespace GhulbusGraphics {
	/** Base interface for all objects in a scene
	 */
	class gbGraphicsObject {
	public:
		/** Let the object draw itself
		 */
		virtual void Render()=0;
		/** Destroy the object and free all acquired ressources
		 */
		virtual void Release()=0;
	protected:
		gbGraphicsObject();
		virtual ~gbGraphicsObject();
		gbGraphicsObject(gbGraphicsObject const&);				///< Private copy constructor (not implemented)
		gbGraphicsObject& operator=(gbGraphicsObject const&);	///< Private copy assignment (not implemented)
	};
	/*-------------------------------------
	 | Extensive Documentation:           |
	 *------------------------------------*/
	/** 
	 * @class gbGraphicsObject
	 *
	 * @details This class defines the base interface shared by all
	 * objects in a scene (or at least those managed by a gbGraphicsManager).
	 * 
	 */
}

#endif
