#include "../include/gbGraphics.hpp"

namespace GhulbusGraphics {
#ifdef _GHULBUSGRAPHICS__USE_D3D9_
	std::map<HWND, gbGraphicsManager_D3D9*>gbGraphicsManager_D3D9::m_CallbackMap;

	LRESULT CALLBACK gbGraphicsManager_D3D9::BootstrapMsgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
	{
		if(msg == WM_NCCREATE) {
			//this is executed prior to window  creation
			CREATESTRUCT* wc = reinterpret_cast<CREATESTRUCT*>(lParam);
			gbGraphicsManager_D3D9* creating_instance = reinterpret_cast<gbGraphicsManager_D3D9*>(wc->lpCreateParams);
			if(!creating_instance) {
				throw( Ghulbus::gbException( Ghulbus::gbException::GB_ILLEGALPARAMETER,
											 "Illegal lparam in bootstrapping callback") );
			}
			creating_instance->m_hWnd = hWnd;
			gbGraphicsManager_D3D9::m_CallbackMap[hWnd] = creating_instance;
#pragma warning(disable: 4311 4312)
			WNDPROC pProc    = (WNDPROC)(&(creating_instance->MsgCallback));
			WNDPROC pOldProc = (WNDPROC)(SetWindowLong(hWnd, GWL_WNDPROC, (LONG)pProc));
#pragma warning(default: 4311 4312)
			if(pOldProc != BootstrapMsgProc) {
				throw( Ghulbus::gbException( Ghulbus::gbException::GB_INVALIDCONTEXT,
											 "Unexpected hook to window callback" ) );
			}
			return pProc(hWnd, msg, wParam, lParam);
		}
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}

	LRESULT CALLBACK gbGraphicsManager_D3D9::MsgCallback(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
	{
		LRESULT ret = 0;
		gbGraphicsManager_D3D9* resp_instance = gbGraphicsManager_D3D9::m_CallbackMap[hWnd];

		if( !(resp_instance->ProcessWindowMessage(hWnd, msg, wParam, lParam, &ret)) ) {
			return DefWindowProc(hWnd, msg, wParam, lParam);
		}
		return ret;
	}

	HWND gbGraphicsManager_D3D9::CreateWin32Window(int winWidth, int winHeight) {
		//create win32 window:
		WNDCLASSEX wc;
		wc.cbSize        = sizeof(WNDCLASSEX);
		wc.style         = CS_HREDRAW | CS_VREDRAW;
		wc.lpfnWndProc   = BootstrapMsgProc;
		wc.cbClsExtra    = 0;
		wc.cbWndExtra    = 0;
		wc.hInstance     = GetModuleHandle(NULL);
		wc.hIcon         = NULL;
		wc.hCursor       = NULL;
		wc.hbrBackground = NULL;
		wc.lpszMenuName  = NULL;
		wc.lpszClassName = m_lpClassName;
		wc.hIconSm       = NULL;

		if(!RegisterClassEx(&wc)) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_FAILED ) );
		}
		//calculate the window border sizes (needed to acquire requested viewport size)
		int xBorderSize = 2*(GetSystemMetrics(SM_CXFIXEDFRAME) + GetSystemMetrics(SM_CXBORDER));
		int yBorderSize = GetSystemMetrics(SM_CYCAPTION) + xBorderSize;
		//notice the this pointer passed as lpParam - this will be
		//processed by the bootstrap callback:
		return( CreateWindow(m_lpClassName, TEXT("D3D_app"),
							 WS_OVERLAPPEDWINDOW, 100, 50, 
							 winWidth+xBorderSize, winHeight+yBorderSize,
							 NULL, NULL, wc.hInstance, this) );
	}

	bool gbGraphicsManager_D3D9::Enumerate(D3DFORMAT format, unsigned int resX, unsigned int resY)
	{
		struct ADAPTERS_T {
			D3DADAPTER_IDENTIFIER9 identifier;
			unsigned int modeCount;
			D3DDISPLAYMODE*	modes;
		} ADAPTERS;
		bool suitableAdapterFound = false;
		//wchar_t buffer[255];
		HRESULT hres;

		unsigned int nAdapters = m_lpD3D->GetAdapterCount();
		//Outer enum-loop over all adapters
		//Loop until either:
		// ~ all adapters were processed
		// ~ a suitable adapter was found
		for(unsigned int i=0; ((i<nAdapters) && (!suitableAdapterFound)); i++) {
			hres = m_lpD3D->GetAdapterIdentifier(i, 0, &(ADAPTERS.identifier));
			if(hres != D3D_OK) {continue;}
			
			ADAPTERS.modeCount = m_lpD3D->GetAdapterModeCount(i, format);

			//sprintf_s(buffer, "Modes available: %i", ADAPTERS[i].modeCount); MessageBox(NULL, buffer, "Enumeration", MB_OK);
			ADAPTERS.modes = new D3DDISPLAYMODE[ADAPTERS.modeCount];
			
			//Inner loop over the modes of one adapter
			// Check for any mode that meets the requirements of format and resolution
			for(unsigned int j=0; j<ADAPTERS.modeCount; j++) {
				m_lpD3D->EnumAdapterModes(i, format, j, &ADAPTERS.modes[j]);
				if( (ADAPTERS.modes[j].Format == format) && 
				    (ADAPTERS.modes[j].Width  == resX)   &&
				    (ADAPTERS.modes[j].Height == resY) ) 
				{
					m_AdapterRecommendation = i;
					suitableAdapterFound = true;
					break; 
				}
			} //mode-loop
			delete[] ADAPTERS.modes;
			//sprintf_s(buffer, "Adapter #%i\n%s", i, ADAPTERS[i].identifier.Description); MessageBox(NULL, buffer, "Enumeration", MB_OK);	
		} //adapter-loop

		return suitableAdapterFound;
	}

	void gbGraphicsManager_D3D9::CreateD3DDevice() {
		memset(&m_d3dpp, 0, sizeof(D3DPRESENT_PARAMETERS));
		m_d3dpp.Windowed         = !m_isFullscreen;
		m_d3dpp.SwapEffect       = D3DSWAPEFFECT_DISCARD;
		m_d3dpp.BackBufferFormat = D3DFMT_A8R8G8B8;
		if(m_isFullscreen) {
			m_d3dpp.BackBufferWidth  = m_scrWidth;
			m_d3dpp.BackBufferHeight = m_scrHeight;
		}
		m_hres = m_lpD3D->CreateDevice( m_AdapterRecommendation, D3DDEVTYPE_HAL, m_hWnd, 
										D3DCREATE_HARDWARE_VERTEXPROCESSING, &m_d3dpp, &m_lpDev );
		if(m_hres != D3D_OK) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_FAILED,
										 "Failed to create D3D device" ) );
		}
	}

	void gbGraphicsManager_D3D9::CreateD3DDrawingSurface() {
		m_hres = m_lpDev->CreateTexture( m_scrWidth, m_scrHeight, 1, 0, D3DFMT_A8R8G8B8, 
										 D3DPOOL_MANAGED, &m_lpSurface, NULL);
		if(m_hres != D3D_OK) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_FAILED,
										 "Unable to create D3D drawing surface" ) );
		}
		m_hres = D3DXCreateSprite(m_lpDev, &m_lpSurfaceSprite);
		if(m_hres != D3D_OK) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_FAILED,
										 "Unable to create D3D drawing sprite" ) );
		}
	}

	gbGraphicsManager_D3D9::gbGraphicsManager_D3D9(int scrWidth, int scrHeight, bool fullscreen)
		:m_hWnd(NULL), m_hres(0), m_hInst(NULL), m_lpD3D(NULL), m_lpDev(NULL), m_lpClassName(NULL),
		 m_AdapterRecommendation(D3DADAPTER_DEFAULT), m_lpSurface(NULL), m_lpSurfaceSprite(NULL)
	{
		//constructors are assumed to be serialized
		size_t n_instances;
		if( (n_instances= gbGraphicsManager_D3D9::m_CallbackMap.size()) > 99) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_FAILED,
										 "Maximum number of parallel instances exceeded" ) );
		}
		m_lpClassName = new wchar_t[32];
		swprintf(m_lpClassName, 32, L"GB3D_WINDOW_%.2d", n_instances);

		int winWidth  = (fullscreen)?(100):(scrWidth);
		int winHeight = (fullscreen)?(100):(scrHeight);
		HWND hTmp = CreateWin32Window(winWidth, winHeight);
		if( (!IsWindow(hTmp)) || (m_hWnd != hTmp) ) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_FAILED,
										 "Failed to create window" ) );
		}
		ShowWindow(m_hWnd, SW_SHOWDEFAULT);
		UpdateWindow(m_hWnd);

		m_lpD3D = Direct3DCreate9( D3D_SDK_VERSION );
		if(!m_lpD3D) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_FAILED,
										 "Failed to initialize D3D" ) );
		}
		if(!Enumerate(D3DFMT_X8R8G8B8, scrWidth, scrHeight)) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_FAILED,
										 "Requested video mode not supported" ) );
		}
		m_scrWidth     = scrWidth;
		m_scrHeight    = scrHeight;
		m_isFullscreen = fullscreen;
		CreateD3DDevice();
		CreateD3DDrawingSurface();
		m_Overlay = new gbTexture_D3D9(m_lpSurface);		//will be deleted by destructor of base class gbGraphicsManager
	}
	
	gbGraphicsManager_D3D9::~gbGraphicsManager_D3D9()
	{
		for(std::vector<gbGraphicsObject*>::iterator i = m_ObjectList.begin(); i != m_ObjectList.end(); ++i) {
			(*i)->Release();
		}
		if(m_lpSurfaceSprite) { m_lpSurfaceSprite->Release();  m_lpSurfaceSprite = NULL; }
		if(m_lpSurface)       {       m_lpSurface->Release();        m_lpSurface = NULL; }
		if(m_lpDev)           {           m_lpDev->Release();            m_lpDev = NULL; }
		if(m_lpD3D)           {           m_lpD3D->Release();            m_lpD3D = NULL; }
		UnregisterClass(m_lpClassName, m_hInst);
		if(m_lpClassName) { delete[] m_lpClassName;  m_lpClassName = NULL; }
	}

	void gbGraphicsManager_D3D9::BeginScene()
	{
		if( m_isRendering || m_Overlay->IsLocked() ) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_INVALIDCONTEXT ) );
		}
		//call D3DDevice::Present() here and not after D3DDevice::EndScene() 
		//to enhance CPU/GPU parallelism
		m_hres = m_lpDev->Present(NULL, NULL, NULL, NULL);
		if(m_hres != D3D_OK) {
			if(m_hres == D3DERR_DEVICELOST) {
				///@todo handling of lost devices
			}
		}
		m_hres = m_lpDev->Clear(0, NULL, D3DCLEAR_TARGET /*| D3DCLEAR_ZBUFFER*/, m_clearcolor, 0.0f, 0);
		m_lpDev->BeginScene();
		m_isRendering = true;
	}

	void gbGraphicsManager_D3D9::EndScene()
	{
		if(!m_isRendering) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_INVALIDCONTEXT ) );
		}

		for(std::vector<gbGraphicsObject*>::iterator i = m_ObjectList.begin(); i != m_ObjectList.end(); ++i) {
			(*i)->Render();
		}

		//draw overlay texture:
		D3DXVECTOR3 pos(    (float)(m_scrWidth/2), (float)(m_scrHeight/2), 0.0f );
		D3DXVECTOR3 center( (float)(m_scrWidth/2), (float)(m_scrHeight/2), 0.0f );
		m_hres = m_lpSurfaceSprite->Begin(D3DXSPRITE_ALPHABLEND);
		m_hres |= m_lpSurfaceSprite->Draw(m_lpSurface, NULL, &center, &pos, 0xFFFFFFFF);
		m_hres |= m_lpSurfaceSprite->End();
		if(m_hres != D3D_OK) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_FAILED,
										 "Error while drawing overlay sprite" ) );
		}

		m_lpDev->EndScene();
		m_isRendering = false;
	}

	void gbGraphicsManager_D3D9::Clear()
	{
		if(!m_isRendering) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_INVALIDCONTEXT ) );
		}
		m_hres = m_lpDev->Clear(0, NULL, D3DCLEAR_TARGET, m_clearcolor, 0.0f, 0);
	}

	void gbGraphicsManager_D3D9::Render(unsigned int dT)
	{
	}

	gbSprite* gbGraphicsManager_D3D9::CreateSprite(int width, int height)
	{
		gbSprite_D3D9* ret;
		gbTexture_D3D9* tTexture;
		LPDIRECT3DTEXTURE9 lpTex;
		LPD3DXSPRITE lpSprite;

		m_hres = D3DXCreateSprite(m_lpDev, &lpSprite);		//will be released by gbSprite::Release()
		if(m_hres != S_OK) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_FAILED,
										 "Sprite creation failed" ) );
		}
		m_hres = m_lpDev->CreateTexture( width, height, 0, D3DUSAGE_AUTOGENMIPMAP, D3DFMT_A8R8G8B8,
										 D3DPOOL_MANAGED, &lpTex, NULL );	//will be released by gbSprite::Release()
		if(m_hres != D3D_OK) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_FAILED,
										 "Texture creation for sprite failed" ) );
		}
		tTexture = new gbTexture_D3D9(lpTex);			//will be deleted by gbSprite destructor
		ret = new gbSprite_D3D9(lpSprite, tTexture);

		m_ObjectList.push_back(ret);
		return ret;
	}

	void gbGraphicsManager_D3D9::ProcessMessages()
	{
		//in win32 messages are processed by callbacks;
		//therefore the actual message handling is done
		//in ProcessWindowMessage()
		MSG msg;
		PeekMessage(&msg, NULL, 0, 0, PM_REMOVE);
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	
	bool gbGraphicsManager_D3D9::ProcessWindowMessage(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam, LRESULT* lResult)
	{
		switch(msg) {
			case WM_KEYDOWN:
				switch(wParam) {
					case VK_ESCAPE:
						PostMessage(hWnd, WM_CLOSE, 0, 0);
						return true;
					default:
						break;
				};
				break;
			case WM_PAINT:
				*lResult = 0;
				return true;
			case WM_DESTROY:
				m_done = true;
				PostQuitMessage(0);
				*lResult = 0;
				return true;
			default:
				break;
		}
		return false;
	}
#endif
}

