#include "../include/gbSprite.hpp"

namespace GhulbusGraphics {
#ifdef _GHULBUSGRAPHICS__USE_SDL_
	gbSprite_SDL::gbSprite_SDL(gbTexture_SDL* texture, int scr_width, int scr_height)
		:m_spriteFaceLL(-1.0, -1.0,  0.0), m_spriteFaceLR( 1.0, -1.0,  0.0), 
		 m_spriteFaceUL(-1.0,  1.0,  0.0), m_spriteFaceUR( 1.0,  1.0,  0.0),
		 m_scrWidth(scr_width), m_scrHeight(scr_height), m_texCoords(1.0, 1.0)
	{
		m_texture = texture;
		SetTexCoords();
		SetPosition(m_position);
	}
	gbSprite_SDL::~gbSprite_SDL() {
		if(m_texture) { delete m_texture;  m_texture = NULL; }
	}
	void gbSprite_SDL::Render() {
		///@todo: too much state calls, optimize!
		glEnable(GL_TEXTURE_2D);
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		glBindTexture(GL_TEXTURE_2D, dynamic_cast<gbTexture_SDL*>(m_texture)->GetTextureId());
		glDisable(GL_DEPTH_TEST);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_BLEND);
		
		glBegin(GL_TRIANGLE_STRIP);
			glColor4f(1.0, 1.0, 1.0, 1.0);
			glTexCoord2f(0.0,                m_texCoords.GetY());  glVertex3fv(m_spriteFaceLL.GetV());
			glTexCoord2f(m_texCoords.GetX(), m_texCoords.GetY());  glVertex3fv(m_spriteFaceLR.GetV());
			glTexCoord2f(0.0,                0.0);                 glVertex3fv(m_spriteFaceUL.GetV());
			glTexCoord2f(m_texCoords.GetX(), 0.0);                 glVertex3fv(m_spriteFaceUR.GetV());
		glEnd();

		glDisable(GL_BLEND);
		glEnable(GL_DEPTH_TEST);
		
	}
	void gbSprite_SDL::Release() {
		if(m_texture->IsLocked()) {
			m_texture->UnlockTexture();
		}
		GLuint id = dynamic_cast<gbTexture_SDL*>(m_texture)->GetTextureId();
		glDeleteTextures(1, &id);
		delete this;
	}

	void gbSprite_SDL::SetPosition(GhulbusMath::gbVector3<float> const& new_pos) {
		///@todo z-coordinate
		///@todo influence of transformation matrices
		m_position = new_pos;
		float left   = -1.0f + (2.0f*new_pos.GetX()                             / static_cast<float>(m_scrWidth)  );
		float right  =  left + (2.0f*static_cast<float>(m_texture->GetWidth())  / static_cast<float>(m_scrWidth)  );
		float top    =  1.0f - (2.0f*new_pos.GetY()                             / static_cast<float>(m_scrHeight) );
		float bottom =  top  - (2.0f*static_cast<float>(m_texture->GetHeight()) / static_cast<float>(m_scrHeight) );
		m_spriteFaceLL.Set( left,  bottom, 0.0f );
		m_spriteFaceLR.Set( right, bottom, 0.0f );
		m_spriteFaceUL.Set( left,  top,    0.0f );
		m_spriteFaceUR.Set( right, top,    0.0f );
	}
	void gbSprite_SDL::SetTexCoords() {
		int texWidth  = dynamic_cast<gbTexture_SDL*>(m_texture)->GetTextureWidth();
		int texHeight = dynamic_cast<gbTexture_SDL*>(m_texture)->GetTextureHeight();
		if(texWidth == m_texture->GetWidth()) {
			m_texCoords.SetX(1.0f);
		} else {
			m_texCoords.SetX( static_cast<float>(m_texture->GetWidth()) / static_cast<float>(texWidth) );
		}
		if(texHeight == m_texture->GetHeight()) {
			m_texCoords.SetY(1.0f);
		} else {
			m_texCoords.SetY( static_cast<float>(m_texture->GetHeight()) / static_cast<float>(texHeight) );
		}
	}
#endif
}

