#include "../include/gbTexture.hpp"

namespace GhulbusGraphics {
#ifdef _GHULBUSGRAPHICS__USE_SDL_
	gbTexture_SDL::gbTexture_SDL(GLuint texture_id, int width, int height, bool npot_supported)
		:m_textureId(texture_id), m_Data(NULL), m_texWidth(0), m_texHeight(0)
	{
		m_width = width;
		m_height = height;
		if(npot_supported) {
			m_texWidth = m_width;
			m_texHeight = m_height;
		} else {
			m_texWidth = GetPOTSize(m_width);
			m_texHeight = GetPOTSize(m_height);
		}
		m_Data = new GBCOLOR[m_texWidth*m_texHeight];
		memset( m_Data, 0, (sizeof(GBCOLOR)*m_texWidth*m_texHeight) );
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, m_texWidth, m_texHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, m_Data);
		if( glGetError() != GL_NO_ERROR ) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_FAILED,
										 "Unable to create OpenGL texture" ) );
		}
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	}

	gbTexture_SDL::~gbTexture_SDL()
	{
		if(m_isLocked) {
			//write back:
			glBindTexture(GL_TEXTURE_2D, m_textureId);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, m_texWidth, m_texHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, m_Data);
		}
		if(m_Data) { delete[] m_Data;  m_Data = NULL; }
	}
	void gbTexture_SDL::LockTexture(GBCOLOR** pData, int* pPitch)
	{
		if(m_isLocked) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_INVALIDCONTEXT ) );
		}
		//copy texture data to buffer:
		glBindTexture(GL_TEXTURE_2D, m_textureId);
		glGetTexImage(GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, m_Data);
		*pData  = m_Data;
		*pPitch = m_texWidth;
		m_isLocked = true;
	}
	void gbTexture_SDL::UnlockTexture()
	{
		if(!m_isLocked) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_INVALIDCONTEXT ) );
		}
		//write back:
		glBindTexture(GL_TEXTURE_2D, m_textureId);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, m_texWidth, m_texHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, m_Data);
		m_isLocked = false;
	}

	GLuint gbTexture_SDL::GetTextureId() {
		if(m_isLocked) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_INVALIDCONTEXT ) );
		}
		return m_textureId;
	}

	int gbTexture_SDL::GetTextureWidth() {
		return m_texWidth;
	}
	int gbTexture_SDL::GetTextureHeight() {
		return m_texHeight;
	}

	int gbTexture_SDL::GetPOTSize(int npot_size) {
		//find the lowest power of two (pot) that
		//is greater or equal the requested size
		int ret;
		//find highest power:
		int max_pot;
		for(max_pot = 0; (npot_size >> max_pot) ;++max_pot) ;
		
		if( ((0x1 << (max_pot-1)) ^ npot_size) == 0) {
			//special case, size is already pot
			ret = npot_size; 
		} else {
			//else, geat the next greater pot
			ret = (0x1 << (max_pot));
		}
		return ret;
	}

#endif
}

