#include "../include/gbTurtle.hpp"

namespace GhulbusGraphics {
	const double gbTurtle::GB2_PI     = 6.283185307179586476925286766559;
	const double gbTurtle::GBPI_180   = 0.017453292519943295769236907684886;
	const double gbTurtle::GB_EPSILON = 1e-5;

	void gbTurtle::SetColor(GBCOLOR_COMPONENT r, GBCOLOR_COMPONENT g, GBCOLOR_COMPONENT b) {
		m_penColor = GBCOLOR32::XRGB(r, g, b);
	}
	void gbTurtle::Go(int units) {
		double oldX = m_posX;
		double oldY = m_posY;
		m_posY += (static_cast<double>(units) * sin(m_angle));
		m_posX += (static_cast<double>(units) * cos(m_angle));
		if(m_penDown) {
			m_gb2D->Line( static_cast<int>(XCord(oldX)),   static_cast<int>(YCord(oldY)), 
						  static_cast<int>(XCord(m_posX)), static_cast<int>(YCord(m_posY)),
						  m_penColor );
		}
	}
	void gbTurtle::TurnLeft(int iAngle) {
		m_angle += static_cast<double>(iAngle) * GBPI_180;
		if(m_angle >= GB2_PI) { m_angle -= GB2_PI; }
		if(m_angle <= GB2_PI) { m_angle += GB2_PI; }
		if(fabs(m_angle) < GB_EPSILON) { m_angle = 0.0; }
	}
	void gbTurtle::TurnRight(int iAngle) {
		m_angle -= static_cast<double>(iAngle) * GBPI_180;
		if(m_angle <= GB2_PI) { m_angle += GB2_PI; }
		if(m_angle >= GB2_PI) { m_angle -= GB2_PI; }
		if(fabs(m_angle) < GB_EPSILON) { m_angle = 0.0; }
	}
	void gbTurtle::SetAngle(int iAngle) {
		m_angle = GBPI_180*static_cast<double>(iAngle);
	}
	void gbTurtle::TurnLeftRad(double fAngle) {
		m_angle += fAngle;
		if(m_angle >= GB2_PI) { m_angle -= GB2_PI; }
		if(m_angle <= GB2_PI) { m_angle += GB2_PI; }
		if(fabs(m_angle) < GB_EPSILON) { m_angle = 0.0; }
	}
	void gbTurtle::TurnRightRad(double fAngle) {
		m_angle -= fAngle;
		if(m_angle <= GB2_PI) { m_angle += GB2_PI; }
		if(m_angle >= GB2_PI) { m_angle -= GB2_PI; }
		if(fabs(m_angle) < GB_EPSILON) { m_angle = 0.0; }
	}
	void gbTurtle::SetAngleRad(double fAngle) {
		m_angle = fAngle;
	}
	void gbTurtle::PenUp() {
		m_penDown = false;
	}
	void gbTurtle::PenDown() {
		m_penDown = true;
	}
	void gbTurtle::PenToggle() {
		m_penDown ^= true;
	}
	bool gbTurtle::IsPenDown() {
		return m_penDown;
	}
	void gbTurtle::Goto(int X, int Y) {
		if(m_penDown) { 
			m_gb2D->Line( static_cast<int>(XCord(m_posX)),     static_cast<int>(YCord(m_posY)), 
						  (static_cast<int>(m_width/2.0) + X), (static_cast<int>(m_height/2.0) - Y), 
						  m_penColor );
		}
		m_posX = static_cast<double>(X);
		m_posY = static_cast<double>(Y);
	}
	void gbTurtle::Reset() {
		m_posX     = 0.0;
		m_posY     = 0.0;
		m_angle    = 0.0;
		m_penDown  = true;
		m_penColor = GBCOLOR32::XRGB(255, 255, 255);
	}
	void gbTurtle::Show() {
		double x = XCord(m_posX);
		double y = YCord(m_posY);
		m_gb2D->Circle( static_cast<int>(x), static_cast<int>(y), 4, GBCOLOR32::XRGB(200, 100, 0) );
		m_gb2D->Line( static_cast<int>(x),                   static_cast<int>(y), 
					  static_cast<int>(x+(10*cos(m_angle))), static_cast<int>(y-(10*sin(m_angle))),
					  GBCOLOR32::XRGB(200, 100, 0) );
		m_gb2D->Circle( static_cast<int>(x+(10*cos(m_angle))), static_cast<int>(y-(10*sin(m_angle))),
						2, GBCOLOR32::XRGB(200, 100, 0) );
	}
	int gbTurtle::GetPosX() {
		return static_cast<int>(m_posX);
	}
	int gbTurtle::GetPosY() {
		return static_cast<int>(m_posY);
	}
	int gbTurtle::GetAngle() {
		return static_cast<int>((1.0 / GBPI_180) * m_angle + 0.5); 
	}
	double gbTurtle::GetAngleRad() {
		return m_angle;
	}

	gbTurtle::gbTurtle(gb2D* lpgb2D) 
		:m_gb2D(lpgb2D), m_posX(0.0), m_posY(0.0), m_penColor(GBCOLOR32::XRGB(255, 255, 255)),
		 m_angle(0.0), m_penDown(true), m_height(0.0), m_width(0.0)
	{
		if(!lpgb2D->IsLocked()) {
			throw( Ghulbus::gbException( Ghulbus::gbException::GB_ILLEGALPARAMETER ) );
		}
		m_width  = m_gb2D->GetWidth();
		m_height = m_gb2D->GetHeight();
	}
	gbTurtle::~gbTurtle() 
	{
	}
}

