//LUA Turtle
// A Lua interface for a LOGO-turtle graphic system
// (C) A. Weis  08-2006
#include "lua_interface.h"
#include <iostream>
#include <fstream>

lua_State* pLua = NULL;

using namespace std;
static const float LTURTLE_VERSION = 1.2f;

const int VIEWPORT_X = 800;
const int VIEWPORT_Y = 600;
const bool FULLSCREEN = false;

void do_pause()
{
#ifdef WIN32
	system("PAUSE");
#else
	cout << "Press RETURN to continue..." << endl;
	cin.get();
#endif
}

int main(int argc, char** argv)
{
	cout.flags(ios_base::showpoint); cout.precision(3);
	cout << "+--------------------------------+\n";
	cout << "[[ Lua Turtle V-" << LTURTLE_VERSION << "\t\t]]\n";
	cout << "+--------------------------------+\n" << endl;

	if(argc < 2) {
		cout << "No input file(s) specified!\n"; do_pause(); return 0;
	} else {
		ifstream lfile(argv[1]);
		if(!lfile)
			{ cout << "Error opening file " << argv[1] << "!\n"; do_pause(); return 11; }
		lfile.close();
	}

	cout << "* Initializing LUA...";
	pLua = luaL_newstate();
	if(!pLua) { cout << "ERROR!"; return 10;}
	lua_gc(pLua, LUA_GCSTOP, 0);  /* stop collector during initialization */
	openstdlibs(pLua);
	lua_gc(pLua, LUA_GCRESTART, 0);
	cout << " done." << endl;

	cout << "The following libraries were loaded: \n    ";
	for(int i=0; lualibs[i].func != NULL; i++) {
		cout << ((i<2)?"":", ") << lualibs[i].name;
	}
	cout << ".\n";

	cout << "* Initializing GB2D...";
	if(!lturtle_Init(VIEWPORT_X, VIEWPORT_Y, FULLSCREEN)) { do_pause(); lua_close(pLua); return 12; }
	cout << " done." << endl;

	cout << "* Starting execution of script " << argv[1] << "...\n";
	cout << "----------------------------------------------------\n" << endl;
	
	if(luaL_dofile(pLua, argv[1])) {
		//report error:
		const char *msg = (lua_isnil(pLua, -1))?NULL:lua_tostring(pLua, -1);
		if (msg != NULL) {
			fprintf(stderr, "%s\n", msg);
			fflush(stderr);
		}
		cout << "\nAn error occured. Exiting...\n"; do_pause(); lua_close(pLua); return 15;
	}

	cout << "\n----------------------------------------------------\n";
	cout << "* Finished." << endl;
	
	cout << "\n* Entering Main Loop." << endl;
	MainLoop();
	cout << "* Main loop finished.\n";

	cout << "* Shutting down LUA...";
	lua_close(pLua);
	cout << " done." << endl;

	cout << "\nGoodbye!" << endl;
	
	do_pause();
	return 0;
}
