#include "lua_interface.h"

#include <iostream>
using namespace std;

static bool isRunning = false;
static GhulbusGraphics::gbGraphicsManager* gbMan=NULL;
static GhulbusGraphics::gb2D* gb2D=NULL;
static GhulbusGraphics::gbTurtle* turtle = NULL;
#ifdef _GHULBUSGRAPHICS__USE_D3D9_
typedef GhulbusGraphics::gbGraphicsManager_D3D9 gbGraphicsManager_in_use;
#else
typedef GhulbusGraphics::gbGraphicsManager_SDL gbGraphicsManager_in_use;
#endif

void Render()						//display callback
{
	if(gbMan->IsDone()) { return; }

	gbMan->BeginScene();
	gbMan->EndScene();
}
bool InitGhulbus(int res_x, int res_y, bool fullscreen)		//Init of display window and gb2d subsystem
{
	//init gbGraphics, gb2D and gbTurtle:
	try {
		gbMan = new gbGraphicsManager_in_use(res_x, res_y, fullscreen);
		gb2D = new GhulbusGraphics::gb2D();
	} catch(Ghulbus::gbException e) {
		cout << "Unable to initialize GB2D subsystem -\n" << e.what() << endl;
		return false;
	}
	
	//open overlay for rendering:
	gb2D->BeginScene( gbMan->GetOverlaySurface() );
	gb2D->Clear();
	turtle = new GhulbusGraphics::gbTurtle(gb2D);
	return true;
}
void MainLoop()
{
	if(!isRunning) { cout << "WARNING! Turtle was not initialized!" << endl; return; }
	//close overlay from rendering:
	gb2D->EndScene();
	if(turtle) { delete turtle; turtle = NULL; }

	while(!gbMan->IsDone())	//Main Loop
	{
		gbMan->ProcessMessages();

		Render();
    }
	//Cleanup:
	if(turtle) { delete turtle; turtle = NULL; }
	if(gb2D)   { delete gb2D; gb2D = NULL; }
	if(gbMan)  { delete gbMan; gbMan = NULL; }
	isRunning = false;
}

int lturtle_Init(int res_x, int res_y, bool fullscreen) 
{
	//Create window:
	if(isRunning) { cout << "Turtle already initialized!" << endl; return 0; }
	if( !InitGhulbus(res_x, res_y, fullscreen) ) { 
		cout << "An error occured. Exiting..." << endl; return 0; 
	}
	isRunning = true;
	return 1;
}

int lturtle_Hello(lua_State* L)
{
	luaL_Buffer ret;
	luaL_buffinit(L, &ret);
	luaL_addstring(&ret, "Hello there :)");
	luaL_pushresult(&ret);
	return 1;
}
int lturtle_Go(lua_State* L) 
{
	lua_Number units = luaL_checknumber(L, 1);
	turtle->Go((int) units);
	lua_Number ret_x = turtle->GetPosX(),  ret_y = turtle->GetPosY();
	lua_pushnumber(L, ret_x);
	lua_pushnumber(L, ret_y);
	return 2;
}
int lturtle_TurnLeft(lua_State* L)
{
	lua_Number angle = luaL_checknumber(L, 1);
	turtle->TurnLeft((int) angle);
	angle = turtle->GetAngle();
	lua_pushnumber(L, angle);
	return 1;
}
int lturtle_TurnRight(lua_State* L)
{
	lua_Number angle = luaL_checknumber(L, 1);
	turtle->TurnRight((int) angle);
	angle = turtle->GetAngle();
	lua_pushnumber(L, angle);
	return 1;
}
int lturtle_Reset(lua_State* L)
{
	turtle->Reset();
	return 0;
}
int lturtle_Show(lua_State* L)
{
	turtle->Show();
	return 0;
}
int lturtle_Goto(lua_State* L)
{
	lua_Number newX = luaL_checknumber(L, 1);
	lua_Number newY = luaL_checknumber(L, 2);
	turtle->Goto((int) newX, (int) newY);
	newX = turtle->GetPosX(); newY = turtle->GetPosY();
	lua_pushnumber(L, newX); lua_pushnumber(L, newY);
	return 2;
}
int lturtle_SetAngle(lua_State* L)
{
	int angle = luaL_checkint(L, 1);
	turtle->SetAngle(angle);
	return 0;
}
int lturtle_PenDown(lua_State* L)
{
	turtle->PenDown();
	return 0;
}
int lturtle_PenUp(lua_State* L)
{
	turtle->PenUp();
	return 0;
}
int lturtle_PenToggle(lua_State* L)
{
	turtle->PenToggle();
	int ret = turtle->IsPenDown();
	lua_pushboolean(L, ret);
	return 1;
}
int lturtle_SetColor(lua_State* L)
{
	int r, g, b;
	int argc = lua_gettop(L);
	switch(argc) 
	{
	case 1:
		{
			const char* color = luaL_checkstring(L, 1);
			if((strcmp(color, "blue")==0) || (strcmp(color, "blau")==0)) {
				turtle->SetColor(0, 0, 255);
			} else if((strcmp(color, "red") == 0) || (strcmp(color, "rot")==0)) {
				turtle->SetColor(255, 0, 0);
			} else if((strcmp(color, "green") == 0) || (strcmp(color, "gruen")==0)) {
				turtle->SetColor(0, 255, 0);
			} else if((strcmp(color, "yellow") == 0) || (strcmp(color, "gelb")==0)) {
				turtle->SetColor(255, 255, 0);
			} else if(strcmp(color, "orange") == 0) {
				turtle->SetColor(255, 128, 0);
			} else if((strcmp(color, "purple") == 0) || (strcmp(color, "violett")==0)) {
				turtle->SetColor(180, 45, 210);
			} else if(strcmp(color, "pink") == 0) {
				turtle->SetColor(250, 60, 180);
			} else if((strcmp(color, "grey") == 0) || (strcmp(color, "grau")==0)) {
				turtle->SetColor(128, 128, 128);
			} else if((strcmp(color, "black") == 0) || (strcmp(color, "schwarz")==0)) {
				turtle->SetColor(0, 0, 0);
			} else {
				turtle->SetColor(255, 255, 255);
			}
		}
		break;
	case 3:
		r = luaL_checkint(L, 1), g = luaL_checkint(L, 2), b = luaL_checkint(L, 3);
		r &= 0xff; g &= 0xff; b &= 0xff;
		turtle->SetColor((unsigned char)r, (unsigned char)g, (unsigned char)b);
		break;
	default:
		turtle->SetColor(255, 255, 255);
		break;
	}
	return 0;
}

LUALIB_API int luaopen_turtle(lua_State* L) {
	luaL_openlib(L, "turtle", lturtlelib, 0);
	return 1;
}

LUALIB_API void openstdlibs (lua_State *L) {
  const luaL_Reg *lib = lualibs;
  for (; lib->func; lib++) {
    lua_pushcfunction(L, lib->func);
    lua_pushstring(L, lib->name);
    lua_call(L, 1, 0);
  }
}
